==> stage-0.7.1.tgz: package/lib/boot.js <==

//
//        Copyright 2010 Johan Dahlberg. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//    1. Redistributions of source code must retain the above copyright notice,
//       this list of conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//
//  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
//  THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
//  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//


var kernel                = require("./kernel");
var available             = kernel.available;
var exists                = kernel.exists;
var connect               = kernel.connect;
var disconnect            = kernel.disconnect;
var dispatch              = kernel.dispatch;
var KernelRef             = kernel.KernelRef;
var kill                  = kernel.kill;

var Actor                 = require("./actor").Actor;


exports.init = function(Ctor, defaults) {
  var cmdparams;
  var first;

  if (Actor.isActorConstructor(Ctor) == false) {
    throw new Error("bad argument, `actor`, expected Actor");
  }

  defaults = defaults || {};

  if (hasCommands(defaults)) {
    cmdparams = defaults;
    defaults = {};
  }

  bindSIGINT();

  require("./rpc").spawn();

  if (cmdparams) {
    executeCommands(cmdparams);
    return;
  }

  kernel.setFirstResponder = true;
  first = new Ctor();

  for (var k in defaults) {
    setStartupParam(k, defaults[k]);
  }

  processStartupOpts(this);
};


function bindSIGINT() {
  process.on("SIGINT", function SIGINT() {
    process.removeListener("SIGINT", SIGINT);
    kernel.exit(kernel.KernelRef, "SIGINT");
    process.on("SIGINT", function() {
      console.error("aborting shutdown process..");
      process.exit(1);
    });
  });
}


function setStartupParam(param, value) {
  var ref;
  var p;
  var m;

  m = /^(\w+)\.(\w+)$|^(\w+)/.exec(param);
  ref = m[1] || KernelRef;
  p = m[2] || m[3];

  process.nextTick(function() {
    if (exists(ref) == false) {
      console.error("invalid option -- %s", param);
      process.exit(1);
    }

    dispatch(ref, "set", [p, value], function(err) {
      if (err) {
        console.error(err.message);
        process.exit(1);
      }
    });
  });
}


function processStartupOpts(self) {
  process.nextTick(function() {
    process.argv.forEach(function(arg) {
      var match;

      if (match = /^--(.+)(?:=(.+))|^--(.+)/.exec(arg)) {
        setStartupParam(match[3] || match[1], match[2] || true);
      }
    });
  });
}

function hasCommands(params) {
  if ((args = getargs([]).slice(2)).length == 0 && params["url"]) {
    return false;
  }
  return true;
}

function executeCommands(params) {
  var inspect = require("util").inspect;
  var handler;
  var args;
  var ref;

  if ((args = getargs([]).slice(2)).length == 0 && params["url"]) {
    return false;
  }

  handler = args.shift();
  ref = connect(params["url"]);

  args = args.map(function(arg) {
    if (arg.substr(0, 5) == "JSON=") {
      return JSON.parse(arg.substr(5));
    }
    return arg;
  });

  dispatch(ref, handler, args, function(err, value) {
    var graph;

    if (err) {
      console.error("error: %s",  (err.message || err));
      kill(KernelRef, 1);
    } else if (arguments.length == 0) {
      console.log("ok");
      kill(KernelRef, 0);
    } else {
      graph = inspect(Array.prototype.slice.call(arguments, 1));
      if (!process.stdout.write(graph + "\n")) {
        process.stdout.on("drain", function() {
          kill(KernelRef, 0);
        });
      } else {
        kill(KernelRef, 0);
      }
    }
  });

  return true;
}

function getargs(defaultValue) {
  var args = process.argv;
  var result = [];
  var arg;

  for (var i = 0, l = args.length; i < l; i++) {
    arg = args[i];
    if (arg[0] != "-") {
      result.push(arg);
    }
  }

  return result.length ? result : defaultValue || null;
};

==> stage-0.7.2.tgz: package/lib/boot.js <==

//
//        Copyright 2010 Johan Dahlberg. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//    1. Redistributions of source code must retain the above copyright notice,
//       this list of conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//
//  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
//  THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
//  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//


var kernel                = require("./kernel");
var available             = kernel.available;
var exists                = kernel.exists;
var connect               = kernel.connect;
var disconnect            = kernel.disconnect;
var dispatch              = kernel.dispatch;
var KernelRef             = kernel.KernelRef;
var kill                  = kernel.kill;

var Actor                 = require("./actor").Actor;


exports.init = function(Ctor, defaults) {
  var cmdparams;
  var first;

  if (Actor.isActorConstructor(Ctor) == false) {
    throw new Error("bad argument, `actor`, expected Actor");
  }

  defaults = defaults || {};

  if (hasCommands(defaults)) {
    cmdparams = defaults;
    defaults = {};
  }

  bindSIGINT();

  require("./rpc").spawn();

  if (cmdparams) {
    executeCommands(cmdparams);
    return;
  }

  kernel.setFirstResponder = true;
  first = new Ctor();

  for (var k in defaults) {
    setStartupParam(k, defaults[k]);
  }

  processStartupOpts(this);
};


function bindSIGINT() {
  process.on("SIGINT", function SIGINT() {
    process.removeListener("SIGINT", SIGINT);
    kernel.exit(kernel.KernelRef, "SIGINT");
    process.on("SIGINT", function() {
      console.error("aborting shutdown process..");
      process.exit(1);
    });
  });
}


function setStartupParam(param, value) {
  var ref;
  var p;
  var m;

  m = /^(\w+)\.(\w+)$|^(\w+)/.exec(param);
  ref = m[1] || KernelRef;
  p = m[2] || m[3];

  process.nextTick(function() {
    if (exists(ref) == false) {
      console.error("invalid option -- %s", param);
      process.exit(1);
    }

    dispatch(ref, "set", [p, value], function(err) {
      if (err) {
        console.error(err.message);
        process.exit(1);
      }
    });
  });
}


function processStartupOpts(self) {
  process.nextTick(function() {
    process.argv.forEach(function(arg) {
      var match;

      if (match = /^--(.+)(?:=(.+))|^--(.+)/.exec(arg)) {
        setStartupParam(match[3] || match[1], match[2] || true);
      }
    });
  });
}

function hasCommands(params) {
  if ((args = getargs([]).slice(2)).length == 0 || !params["url"]) {
    return false;
  }
  return true;
}

function executeCommands(params) {
  var inspect = require("util").inspect;
  var handler;
  var args;
  var ref;

  if ((args = getargs([]).slice(2)).length == 0 || !params["url"]) {
    return false;
  }

  handler = args.shift();
  ref = connect(params["url"]);

  args = args.map(function(arg) {
    if (arg.substr(0, 5) == "JSON=") {
      return JSON.parse(arg.substr(5));
    }
    return arg;
  });

  dispatch(ref, handler, args, function(err, value) {
    var graph;

    if (err) {
      console.error("error: %s",  (err.message || err));
      kill(KernelRef, 1);
    } else if (arguments.length == 0) {
      console.log("ok");
      kill(KernelRef, 0);
    } else {
      graph = inspect(Array.prototype.slice.call(arguments, 1));
      if (!process.stdout.write(graph + "\n")) {
        process.stdout.on("drain", function() {
          kill(KernelRef, 0);
        });
      } else {
        kill(KernelRef, 0);
      }
    }
  });

  return true;
}

function getargs(defaultValue) {
  var args = process.argv;
  var result = [];
  var arg;

  for (var i = 0, l = args.length; i < l; i++) {
    arg = args[i];
    if (arg[0] != "-") {
      result.push(arg);
    }
  }

  return result.length ? result : defaultValue || null;
};
